var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { css, CSSResult, html, LitElement } from 'lit';
import { customElement, property, query, state } from 'lit/decorators.js';
import { Bubble } from '../utils/bubble.js';
import { cssReset } from '../utils/css-reset.js';
/** Editable single line of text that grows to fit content. */
let LineInput = class LineInput extends LitElement {
    static { this.styles = css `
    ${cssReset}

    ::selection {
      background-color: var(--shade-bright-yellow);
    }

    p {
      margin-block: 0;
      /* Stagger bounding box to allow slotted button to float. */
      display: inline;
      /* break words if needed*/
      overflow-wrap: break-word;
    }
    p::before {
      /* force minimum height for empty text areas */
      float: left;
      content: '';
      display: block;
      min-height: 15.5px;
    }

    p:not(:focus) {
      /* Disable text selection which may interfere with dragging. */
      user-select: none;
    }

    p:focus {
      cursor: text;
      /* Disable Chrome and Firefox focus outline which is indicated by higher
         level elements. */
      outline-style: none;
    }
  `; }
    /**
     * Text render state. Undefined allows initial update since text is always a
     * string.
     */
    #text;
    constructor() {
        super();
        this.context = {};
        this._spellcheck = false;
        this.#onClick = () => {
            // pointerdown default is prevented by event-translator.
            this._p.focus();
        };
        this.addEventListener('click', this.#onClick);
    }
    render() {
        if (this.line && this.context.focus?.line === this.line) {
            requestAnimationFrame(() => {
                this._p.focus();
                if (this.context.focus?.startOfLine) {
                    this.getSelection()?.setPosition(this._p, 0);
                }
            });
        }
        return html `
      <p
        contenteditable="true"
        .innerText=${this.#text ?? ''}
        role="textbox"
        spellcheck="${this._spellcheck}"
        tabindex="0"
        @beforeinput=${this.#onBeforeInput}
        @blur=${this.#onBlur}
        @click=${this.#onClick}
        @dragstart=${this.#onDragStart}
        @focus=${this.#onFocus}
        @keyup=${this.#onKeyUp}
        @input=${this.#onInput}
      ></p>
      <slot></slot>
    `;
    }
    shouldUpdate(props) {
        const update = super.shouldUpdate(props);
        if (this.line?.text === this.#text && !props.has('_spellcheck'))
            return false;
        this.#text = this.line?.text;
        return update;
    }
    getSelection() {
        // globalThis fallback for Firefox which pierces shadow DOM.
        return ((this.renderRoot).getSelection?.() ?? getSelection());
    }
    #getCursorPosition() {
        return this.getSelection()?.getRangeAt(0).startOffset ?? 0;
    }
    #onBeforeInput(ev) {
        // https://w3c.github.io/input-events/#interface-InputEvent-Attributes
        // Removals (remove content) and un/redo (valid content) are allowed.
        if (/^(delete|history)/.test(ev.inputType))
            return;
        // All other edits disabled.
        ev.preventDefault();
        if (/insert(?:LineBreak|Paragraph)/.test(ev.inputType)) {
            this.dispatchEvent(Bubble('break-text', this.#getCursorPosition()));
            return;
        }
        // Ignore if not an insertion (add content).
        if (!/^insert/.test(ev.inputType))
            return;
        // to-do: map insertOrderedList and insertUnorderedList to multiple lines?
        //   const remove =
        //   (ev.key === 'Backspace' || ev.key === 'Delete') && emptyGroup
        // if (ev.key !== 'Enter' && !remove) return
        const data = ev.data ?? ev.dataTransfer?.getData('text/plain');
        const input = data?.replaceAll(/\r?\n/g, '');
        // Trigger #onInput().
        if (input) {
            this._p.focus();
            document.execCommand('insertText', false, input);
        }
    }
    #onBlur() {
        // to-do: should this be this.getSelection()?
        getSelection()?.empty(); // Clear any selected text
        this._spellcheck = false;
        if (this.line)
            this.dispatchEvent(Bubble('blur-line', this.line));
    }
    #onClick;
    #onDragStart(ev) {
        // Disable text dragging; this is confusing in a small space with line
        // dragging.
        ev.preventDefault();
    }
    #onFocus() {
        this._spellcheck = true;
        if (this.line)
            this.dispatchEvent(Bubble('focus-line', this.line));
        if (this.context.focus?.startOfLine)
            return;
        const sel = getSelection();
        if (!sel)
            return;
        // for tabbing and pointer down; set to eol.
        const range = document.createRange();
        range.selectNodeContents(this._p);
        range.collapse(false);
        sel.removeAllRanges();
        sel.addRange(range);
    }
    #onKeyUp(ev) {
        if (ev.key === 'Escape')
            this.blur();
    }
    #onInput() {
        this.#text = this._p.textContent ?? '';
        this.dispatchEvent(Bubble('edit-text', this.#text));
    }
};
__decorate([
    property({ attribute: false })
], LineInput.prototype, "context", void 0);
__decorate([
    property({ attribute: false })
], LineInput.prototype, "line", void 0);
__decorate([
    query('p')
], LineInput.prototype, "_p", void 0);
__decorate([
    state()
], LineInput.prototype, "_spellcheck", void 0);
LineInput = __decorate([
    customElement('line-input')
], LineInput);
export { LineInput };
//# sourceMappingURL=line-input.js.map