var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { css, CSSResult, html, LitElement } from 'lit';
import { customElement, property } from 'lit/decorators.js';
import faviconUnsavedURI from '../../favicon/favicon-unsaved.svg';
import faviconURI from '../../favicon/favicon.svg';
import { isFileModified, openFile, reopenFile, saveFile } from '../../io/file.js';
import { loadStorage, saveStorage } from '../../io/storage.js';
import { TextTree } from '../../tree/text-tree.js';
import { cssReset } from '../../utils/css-reset.js';
import '../app-menu/app-menu.js';
import '../group-list.js';
import '../split-block.js';
import '../text-editor.js';
import { linearTextVars } from './linear-text-vars.js';
let LinearText = class LinearText extends LitElement {
    static { this.styles = css `
    ${cssReset}
    ${linearTextVars}

    :host {
      display: flex;
      flex-direction: column;
      display: block;

      width: 100%;
      height: 100%;
      overflow: hidden;

      user-select: none;

      font-family: sans-serif;
      font-size: 16px;
      line-height: 1.1;

      background: #006400;
      background-color: black;
    }

    app-menu {
      position: fixed;
      left: 50%;
      bottom: 0;
      translate: -50%;
      /* need spacer so you can click notes at the bottom underneath */
    }

    split-block {
      flex-grow: 1;
      /* to-do: fix with divider which makes this look weird */
      border-width: 1px;
      border-style: solid;
      border-radius: 8px;
      border-color: black;
    }

    group-list {
      display: block;
      width: 100%;
      height: 100%;
    }
  `; }
    #ctx = {};
    #fileAndHandle;
    #filename;
    /** True if no unsaved changes. */
    #saved = false;
    #timer;
    constructor() {
        super();
        console.log(`Linear Text v${linearTextVersion} by ──oidoid>°─`);
        const save = loadStorage();
        if (save) {
            this.#saved = save.saved;
            this.tree = TextTree(save.text, 2); // to-do: pass indent
            this.#filename = save.filename;
            this.#updateTitle();
        }
        else
            this.tree = TextTree('hello', 2);
    }
    render() {
        return html `
      <split-block
        ><group-list
          slot="lhs"
          .context=${this.#ctx}
          .tree=${this.tree}
          @blur-line=${this.#onBlurLine}
          @break-line=${this.#onBreakLine}
          @drag-line-end=${this.#onDragLineEnd}
          @edit-line=${this.#onEditLine}
          @expand-line=${this.#onExpandLine}
          @focus-line=${this.#onFocusLine}
          @remove-line=${this.#onRemoveLine}
        ></group-list
        ><text-editor
          slot="rhs"
          text="${TextTree.toString(this.tree)}"
          @edit-text=${this.#onEditText}
        ></text-editor></split-block
      ><app-menu
        @new-file=${this.#onNewFile}
        @open-file=${this.#onOpenFile}
        @save-file=${this.#onSaveFile}
        @save-file-as=${this.#onSaveFileAs}
      ></app-menu>
    `;
    }
    #onBlurLine(ev) {
        if (this.#ctx.focus?.line === ev.detail)
            delete this.#ctx.focus;
    }
    #onBreakLine(ev) {
        // to-do: always return the relevant node and let caller walk up tree
        const next = TextTree.breakLine(ev.detail.line, ev.detail.at, 'LF'); // to-do: pass line ending
        this.#ctx.focus = { line: next.new, startOfLine: true };
        this.#updateTreeWithUnsavedChanges(next.root);
    }
    #updateTreeWithUnsavedChanges(tree) {
        this.tree = tree;
        this.#saved = false;
        this.#updateTitle();
    }
    #updateTitle() {
        // if (this.#saved !== saved || ) to-do: accept saved state and file handle and test for title save / diff
        setTitle(this.#filename, this.#saved);
    }
    // @add-line=${this.#onAddLine}
    //   #onAddLine(ev: CustomEvent<{ el: LitElement; group: Group }>): void {
    //   this.tree = TextTree.addLine(ev.detail.group, 'End', 'LF') // to-do: pass newline preference. Try to use first line in file but fallback to pref.
    // }
    #onEditText(ev) {
        this.#updateTreeWithUnsavedChanges(TextTree(ev.detail, 2)); // to-do: pass in indent width
    }
    #onDragLineEnd(ev) {
        this.#updateTreeWithUnsavedChanges(TextTree.moveLine(ev.detail.from, ev.detail.to, ev.detail.at));
    }
    // @line-drag-over-group=${this.#onDragLineOverGroup}
    // #onDragLineOverGroup(
    //   _ev: CustomEvent<{ el: LitElement; group: Group }>,
    // ): void {
    //   // this.tree = textTreeMoveLine(this.#drag.line, ev.detail.group, 'End')
    // }
    #onEditLine(ev) {
        this.#updateTreeWithUnsavedChanges(TextTree.setLineText(ev.detail.line, ev.detail.text));
    }
    #onExpandLine(ev) {
        this.tree = TextTree.setLineExpand(ev.detail.line, ev.detail.expand);
    }
    #onFocusLine(ev) {
        this.#ctx.focus = {
            line: ev.detail,
            startOfLine: this.#ctx.focus?.line === ev.detail
                ? this.#ctx.focus.startOfLine
                : false
        };
    }
    connectedCallback() {
        super.connectedCallback();
        addEventListener('focus', this.#onReloadFile);
        this.#timer = setInterval(
        // to-do: only save unsaved? Saving every time possibly blows away failed
        // to parse data but not saving everytime allows stale files.
        () => saveStorage(this.#filename, this.#saved, this.tree), 30_000);
    }
    disconnectedCallback() {
        clearInterval(this.#timer);
        removeEventListener('focus', this.#onReloadFile);
        super.disconnectedCallback();
    }
    #onReloadFile = async () => {
        if (!this.#saved || !this.#fileAndHandle)
            return;
        // If we have a handle, we can check the timestamp of the current snapshot
        // against a new opened snapshot to determine whether to reload.
        const newFileAndHandle = await reopenFile(this.#fileAndHandle);
        if (!isFileModified(this.#fileAndHandle, newFileAndHandle))
            return;
        this.#fileAndHandle = newFileAndHandle;
        const str = await this.#fileAndHandle.file.text();
        const textStr = TextTree.toString(this.tree);
        if (str !== textStr)
            this.tree = TextTree(str, 2); // this invalidates the entire board--all IDs are lost // to-do: accept indentw
    };
    #onRemoveLine(ev) {
        this.#updateTreeWithUnsavedChanges(TextTree.removeLine(ev.detail.line));
    }
    async #onSaveFile() {
        const text = TextTree.toString(this.tree);
        this.#fileAndHandle = await saveFile(this.#fileAndHandle, text);
        this.#filename = this.#fileAndHandle?.file.name;
        this.#saved = !!this.#fileAndHandle;
        this.#updateTitle();
    }
    async #onSaveFileAs() {
        const text = TextTree.toString(this.tree);
        this.#fileAndHandle = await saveFile(undefined, text);
        this.#filename = this.#fileAndHandle?.file.name;
        this.#saved = !!this.#fileAndHandle;
        this.#updateTitle();
    }
    #onNewFile() {
        this.#fileAndHandle = undefined;
        this.#filename = undefined;
        this.tree = TextTree('', 2); // to-do: accept indentw
        this.#saved = true;
        this.#updateTitle();
    }
    async #onOpenFile() {
        this.#fileAndHandle = await openFile();
        this.#filename = this.#fileAndHandle?.file.name;
        if (!this.#fileAndHandle)
            return;
        const str = await this.#fileAndHandle.file.text();
        this.tree = TextTree(str, 2); // to-do: accept indentw
        this.#saved = true;
        this.#updateTitle();
    }
};
__decorate([
    property({ attribute: false })
], LinearText.prototype, "tree", void 0);
LinearText = __decorate([
    customElement('linear-text')
], LinearText);
export { LinearText };
function setTitle(filename, saved) {
    document.title = saved
        ? filename == null
            ? 'Linear Text'
            : `${filename} – Linear Text`
        : filename == null
            ? '• (Unsaved) – Linear Text'
            : `• ${filename} (Unsaved) – Linear Text`;
    const favicon = document.querySelector('link[rel="icon"]');
    if (favicon)
        favicon.href = saved ? faviconURI : faviconUnsavedURI;
}
//# sourceMappingURL=linear-text.js.map