var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { css, CSSResult, html, LitElement } from 'lit';
import { customElement, property, query } from 'lit/decorators.js';
import { classMap } from 'lit/directives/class-map.js';
import { Bubble } from '../utils/bubble.js';
import { cssReset } from '../utils/css-reset.js';
import './line-input.js';
import { LineInput } from './line-input.js';
import './line-list.js';
let NoteCard = class NoteCard extends LitElement {
    constructor() {
        super(...arguments);
        this.context = {};
    }
    static { this.styles = css `
    ${cssReset}

    :host {
      contain: content;
      --indent: 16px;
    }

    .note {
      border-radius: 1px;
      border-width: 0.5px;
      border-color: #888;
      border-style: solid;

      padding-top: 8px;
      padding-right: 6px;
      padding-bottom: 6px;
      padding-left: 6px;

      background-color: var(--shade-yellow);
      color: var(--color, --shade-black);

      /* Show a pointing finger for the whole note except input. */
      cursor: grab;

      min-height: 24px;
      width: var(--note-card-width);
    }

    .input-wrapper {
      cursor: pointer;
    }
    .input-wrapper:focus-within {
      cursor: text;
    }
    /* display inline on p, make dogear separate inline el */

    .note:hover,
    .note:focus-within,
    :host(.ghost) > .note {
      background-color: rgb(255, 255, 215);
    }

    :host(.picked) {
      opacity: 0.3;
      --color: #0000; /* Hide text on self and child notes. */
    }

    .note:focus-within {
      border-color: black;
    }

    button {
      background-color: #0000;
      border-style: none;
      cursor: pointer;
      color: inherit;
      padding-inline: 2px;
      padding-block: 0; /* Avoid subtle layout shift for new parents. */
    }

    line-input {
      cursor: pointer;
    }

    .note::after {
      content: '';
      display: block;
      clear: both;
    }

    .hidden {
      display: none;
    }

    button:hover {
      /* to-do: vars */
      background-color: #0001;
    }

    button {
      float: inline-end;
    }

    .input-wrapper::after {
      content: '';
      display: block;
      clear: both;
    }

    line-list:not([data-expand]) {
      /* Hide collapsed children. */
      display: none;
    }
  `; }
    render() {
        return html `
      <div class="note">
        <div class="input-wrapper" @click=${this.#onClickTextArea}>
          <line-input
            .context=${this.context}
            .line=${this.line}
            @blur=${this.#onBlur}
            @break-text=${this.#onBreakText}
            @edit-text=${this.#onEditText}
            ><button
              class=${classMap({ hidden: !this.line?.down.length })}
              @click=${this.#onToggleExpand}
              title="Toggle sub-lines."
            >
              …
            </button></line-input
          >
        </div>
      </div>
      <line-list
        .context=${this.context}
        ?data-expand=${!this.line?.down.length || this.line?.expand}
        .list=${this.line}
      ></line-list>
    `;
    }
    get rect() {
        return this.renderRoot.querySelector('.note').getBoundingClientRect();
    }
    #onClickTextArea() {
        this._input.click();
    }
    #onToggleExpand(ev) {
        ev.stopPropagation(); // Don't leak to click on text.
        if (!this.line)
            return;
        this.dispatchEvent(Bubble('expand-line', {
            line: this.line,
            expand: !this.line?.expand
        }));
    }
    // to-do: this should be on the line-input i think
    #onBlur() {
        if (!this.line || this.line.text || this.line.down.length)
            return;
        this.dispatchEvent(Bubble('remove-line', { line: this.line }));
    }
    // to-do: pass line to lint-input and move this logic there?
    #onBreakText(ev) {
        if (!this.line)
            return;
        this.dispatchEvent(Bubble('break-line', { line: this.line, at: ev.detail }));
    }
    #onEditText(ev) {
        if (!this.line)
            return;
        this.dispatchEvent(Bubble('edit-line', { line: this.line, text: ev.detail }));
    }
};
__decorate([
    property({ attribute: false })
], NoteCard.prototype, "context", void 0);
__decorate([
    property({ attribute: false })
], NoteCard.prototype, "line", void 0);
__decorate([
    query('line-input')
], NoteCard.prototype, "_input", void 0);
NoteCard = __decorate([
    customElement('note-card')
], NoteCard);
export { NoteCard };
//# sourceMappingURL=note-card.js.map