var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { css, CSSResult, html, LitElement } from 'lit';
import { customElement, property, query } from 'lit/decorators.js';
import { Bubble } from '../utils/bubble.js';
import { cssReset } from '../utils/css-reset.js';
import { throttle } from '../utils/throttle.js';
let TextEditor = class TextEditor extends LitElement {
    constructor() {
        super(...arguments);
        this.text = '';
        this.#selectionStart = 0;
        this.#selectionEnd = 0;
        this.#scrollTopy = 0;
    }
    static { this.styles = css `
    ${cssReset}

    ::selection {
      background-color: var(--shade-bright-yellow);
    }
    :host {
      min-height: 100%;
    }
    textarea {
      /* Prevent scroll. */
      display: block;

      /* Fill block. */
      min-height: 100%;
      width: 100%;

      /* Disable resizable default. */
      resize: none;

      /* Only break on newline. */
      white-space: pre;
    }
  `; }
    #selectionStart;
    #selectionEnd;
    #scrollTopy;
    willUpdate(props) {
        super.willUpdate(props); // super?
        if (!this._textArea)
            return;
        // this sucks because i want xy but get in characters
        this.#selectionStart = this._textArea.selectionStart;
        this.#selectionEnd = this._textArea.selectionEnd;
        this.#scrollTopy = this._textArea.scrollTop;
    }
    updated(props) {
        super.updated(props);
        this._textArea.selectionStart = this.#selectionStart;
        this._textArea.selectionEnd = this.#selectionEnd;
        this._textArea.scrollTop = this.#scrollTopy;
    }
    render() {
        return html `
      <textarea
        @beforeinput=${this.#onBeforeInput}
        @input=${throttle(this.#onInput, 100)}
        spellcheck
        .value=${this.text}
      ></textarea>
    `;
    }
    #onBeforeInput(ev) {
        if (ev.inputType === 'insertLineBreak') {
            // Workaround unintuitive Chromium scroll behavior by manually inserting
            // a line break; https://stackoverflow.com/questions/56329625.
            ev.preventDefault();
            document.execCommand('insertText', false, '\n');
            this.#onInput();
        }
    }
    #onInput() {
        this.text = this._textArea?.value ?? '';
        this.dispatchEvent(Bubble('edit-text', this.text));
    }
};
__decorate([
    property()
], TextEditor.prototype, "text", void 0);
__decorate([
    query('textarea')
], TextEditor.prototype, "_textArea", void 0);
TextEditor = __decorate([
    customElement('text-editor')
], TextEditor);
export { TextEditor };
//# sourceMappingURL=text-editor.js.map