import { fileOpen, fileSave } from 'browser-fs-access';
const defaultExtension = '.text';
const defaultMimeType = 'text/plain';
const defaultExtensions = [defaultExtension, '.md', '.txt'];
export async function openFile() {
    let file;
    try {
        file = await fileOpen({
            mimeTypes: [defaultMimeType],
            extensions: [...defaultExtensions],
            description: 'Plain Text Files'
        });
    }
    catch (err) {
        if (isCanceledByUser(err))
            return;
        throw err;
    }
    return { file, handle: file.handle };
}
export async function reopenFile(fileAndHandle) {
    if (!fileAndHandle.handle)
        return fileAndHandle;
    return {
        file: await fileAndHandle.handle.getFile(),
        handle: fileAndHandle.handle
    };
}
export function isFileModified(before, after) {
    return before.file.lastModified < after.file.lastModified;
}
export async function saveFile(fileAndHandle, text) {
    let handle;
    try {
        handle = await fileSave(new Blob([text], { type: defaultMimeType }), {
            fileName: fileAndHandle?.file.name ?? `untitled${defaultExtension}`,
            extensions: [...defaultExtensions]
        }, fileAndHandle?.handle);
    }
    catch (err) {
        if (isCanceledByUser(err)) {
            // Don't discard existing handle.
            return fileAndHandle;
        }
        throw err;
    }
    if (!handle)
        return fileAndHandle;
    return { file: await handle.getFile(), handle };
}
function isCanceledByUser(err) {
    if (!(err instanceof DOMException))
        return false;
    return (
    // Canceled from system file picker.
    err.name === 'AbortError' ||
        // Canceled from browser file save prompt.
        err.name === 'NotAllowedError');
}
//# sourceMappingURL=file.js.map