/** n-ary tree. Subtypes may specify nonnullish linkage if treeAdd() is used. */
export type Tree = {
    down?: undefined | Tree[];
    up?: Tree | undefined;
};
/** Tree subtype with optional linkage. */
export type OptTree<T> = Omit<T, 'down' | 'up'> & Tree;
/**
 * Add node to up.down[at]. A negative index counts back from the first item.
 * Returns node.
 */
export declare function treeAdd<T extends Tree>(node: OptTree<T>, up: T['up'], at: number): T;
/** Disconnect node from up. Return node. */
export declare function treeRemove<T extends Tree>(node: T): T;
export declare function treeRoot<T extends Tree>(node: Tree): T;
/**
 * Shallow-copy every node from node upwards. Returns the node copy.
 *
 * Mutating any node in an immutable up-down tree would replace every node, even
 * an immutable down tree would replace every up node.
 */
export declare function treeTouch<T extends Tree>(node: Readonly<OptTree<T>>): T;
