/**
 * Add node to up.down[at]. A negative index counts back from the first item.
 * Returns node.
 */
export function treeAdd(node, up, at) {
    node.up = up;
    node.down ??= []; // Allow nonnullish linkage.
    if (!up)
        return node;
    up.down ??= [];
    up.down.splice(at < 0 ? up.down.length + 1 + at : at, 0, node);
    return node;
}
/** Disconnect node from up. Return node. */
export function treeRemove(node) {
    if (!node.up)
        return node;
    const i = node.up.down?.indexOf(node) ?? -1;
    if (i === -1)
        throw Error('up missing down tree');
    node.up.down.splice(i, 1);
    return node;
}
export function treeRoot(node) {
    while (node.up)
        node = node.up;
    return node;
}
/**
 * Shallow-copy every node from node upwards. Returns the node copy.
 *
 * Mutating any node in an immutable up-down tree would replace every node, even
 * an immutable down tree would replace every up node.
 */
export function treeTouch(node) {
    const touched = { ...node };
    for (const down of node.down ?? [])
        down.up = touched;
    if (!node.up)
        return touched;
    const i = node.up.down?.indexOf(node) ?? -1;
    if (i === -1)
        throw Error('up missing down tree');
    node.up.down[i] = touched;
    treeTouch(node.up);
    return touched;
}
//# sourceMappingURL=tree.js.map